from xl_word.sheet import Sheet
from pathlib import Path


class Document(Sheet):
    """Word表单对象，用于处理组件化的Word文档渲染"""

    # 自定义语法映射到Jinja2语法
    SYNTAX_MAP = {
        r'($': '{%',
        r'$)': '%}', 
        r'((': '{{',
        r'))': '}}',
    }

    def __init__(self, tpl_path, component_folder=None, xml_folder=None):
        super().__init__(tpl_path, xml_folder)
        self.component_folder = component_folder

    def _get_component_files(self):
        """获取所有组件XML文件"""
        if not self.component_folder:
            return []
        return [f for f in Path(self.component_folder).rglob('*.xml')]

    def _build_component_template(self):
        """构建组件模板字符串
        
        Args:
            component_files: 组件文件列表
            
        Returns:
            str: 组合后的模板字符串
        """
        component_files = self._get_component_files()

        template_parts = ['($ for item in data $)']
        
        for index, filepath in enumerate(component_files):
            component_type = filepath.stem
            component_content = self._read_component_file(filepath)
            condition = 'if' if index == 0 else 'elif'
            template_parts.append(
                f"($ {condition} item['component']=='{component_type}' $){component_content}"
            )

        template_parts.extend(['($ endif $)', '($ endfor $)'])
        return ''.join(template_parts)

    def _read_component_file(self, filepath):
        """读取组件文件内容
        
        Args:
            filepath: 组件文件路径
            
        Returns:
            str: 组件文件内容
        """
        with open(filepath, 'r', encoding='utf-8') as file:
            return file.read()

    def _convert_syntax(self, content):
        """转换自定义语法为Jinja2语法
        
        Args:
            content: 包含自定义语法的内容
            
        Returns:
            str: 转换后的内容
        """
        for custom, jinja in self.SYNTAX_MAP.items():
            content = content.replace(custom, jinja)
        return content

    def render(self, data):
        """渲染文档
        
        Args:
            data: 渲染数据
        """
        template_xml = self._build_component_template()
        
        document_xml = self.render_xml('document', dict(document=template_xml)).decode()
        document_xml = self._convert_syntax(document_xml)
        
        self['word/document.xml'] = document_xml.encode('utf-8')
        super().render(data)



from xl_word.tool.gui import *
from xl_word.tool.utility import SuperWordFile
import os


class WordTemplateEditor:
    """Word模板编辑器"""

    def __init__(self):
        """初始化编辑器"""
        self.app = App({
            'title': 'XiLong DOCX Toolkit',
            'size': (420, 330),
            'loc': (500, 300)
        })
        self.root = self.app.instance
        self._init_ui()

    def _init_ui(self):
        """初始化UI界面"""
        self.main_frame = Frame(self.root, relief='ridge', borderwidth=1)
        self.main_frame.pack(fill=BOTH, expand=True)
        self._create_input_fields()
        self._create_buttons()

    def _create_input_fields(self):
        """创建输入字段"""
        # 创建拖拽区域
        self.drop_label = Label(self.main_frame, text="拖拽DOCX文件到这里", relief="solid")
        self.drop_label.pack()
        self.drop_label.place(x=30, y=30, width=350, height=40)
        
        # 绑定拖拽事件
        self.drop_label.drop_target_register("DND_Files")
        self.drop_label.dnd_bind('<<Drop>>', self._on_drop)

    def _create_buttons(self):
        """创建所有按钮"""
        # 提取按钮组
        extract_buttons = [
            ('提取document', self._extract_document, 30),
            ('提取header', self._extract_header, 155),
            ('提取footer', self._extract_footer, 280)
        ]
        
        for text, command, x in extract_buttons:
            btn = self.app.button(
                self.main_frame, text, command, width=13
            )
            btn.pack()
            btn.place(x=x, y=90)

        # 转换按钮组
        convert_buttons = [
            ('DOCX转XML(竖向)', self._word2xml_v, 30, 150),
            ('DOCX转XML(横向)', self._word2xml_h, 220, 150),
            ('XML转DOCX(竖向)', self._xml2word_v, 30, 190),
            ('XML转DOCX(横向)', self._xml2word_h, 220, 190)
        ]

        for text, command, x, y in convert_buttons:
            btn = self.app.button(
                self.main_frame, text, command, width=20
            )
            btn.pack()
            btn.place(x=x, y=y)

    def _on_drop(self, event):
        """处理文件拖拽"""
        file_path = event.data
        if file_path.endswith('.docx'):
            self.current_file = file_path
            self.drop_label.config(text=f"当前文件: {os.path.basename(file_path)}")
        else:
            self.app.alert("错误", "请拖拽DOCX文件(.docx)")

    # DOCX文件操作方法
    def _get_edit_input(self):
        """获取编辑输入"""
        if not hasattr(self, 'current_file'):
            self.app.alert("错误", "请先拖拽DOCX文件")
            return None, None
            
        folder = os.path.dirname(self.current_file)
        file = os.path.basename(self.current_file)
        return folder, file

    def _extract_document(self):
        """提取document.xml"""
        folder, file = self._get_edit_input()
        if folder and file:
            SuperWordFile(folder, file).extract('document.xml')

    def _extract_header(self):
        """提取header.xml"""
        folder, file = self._get_edit_input()
        if folder and file:
            SuperWordFile(folder, file).extract('header.xml')

    def _extract_footer(self):
        """提取footer.xml"""
        folder, file = self._get_edit_input()
        if folder and file:
            SuperWordFile(folder, file).extract('footer.xml')

    def _word2xml_h(self):
        """DOCX转XML(横向)"""
        SuperWordFile.word2xml('h')

    def _word2xml_v(self):
        """DOCX转XML(竖向)"""
        SuperWordFile.word2xml('v')

    def _xml2word_h(self):
        """XML转DOCX(横向)"""
        SuperWordFile.xml2word('h')

    def _xml2word_v(self):
        """XML转DOCX(竖向)"""
        SuperWordFile.xml2word('v')

    def run(self):
        """运行编辑器"""
        self.app.run()


if __name__ == '__main__':
    editor = WordTemplateEditor()
    editor.run()


from xl_word.word_file import WordFile
from jinja2 import Template
from pathlib import Path
import math 


class Sheet(WordFile):
    """Word表单对象，用于处理Word模板文件的渲染和XML操作"""

    TEMPLATE_FUNCTIONS = {
        'enumerate': enumerate,
        'len': len, 
        'isinstance': isinstance,
        'tuple': tuple,
        'list': list,
        'str': str,
        'float': float,
        'int': int,
        'range': range,
        'ceil': math.ceil,
        'type': type
    }

    def __init__(self, tpl_path, xml_folder=None):
        """初始化Sheet对象
        
        Args:
            tpl_path: Word模板文件路径
            xml_folder: 可选的XML模板文件夹路径
        """
        super().__init__(tpl_path)
        self.xml_folder = xml_folder

    def render_template(self, template_content, data):
        """渲染Jinja2模板
        
        Args:
            template_content: 模板内容字符串
            data: 用于渲染的数据字典
            
        Returns:
            str: 渲染后的内容
        """
        rendered = Template(template_content).render(
            **data, 
            **self.TEMPLATE_FUNCTIONS
        )
        return rendered.replace(' & ', ' &amp; ')

    def get_xml_template(self, xml_filename):
        """获取XML模板内容
        
        Args:
            xml_filename: XML文件名(不含扩展名)
            
        Returns:
            str: XML模板内容
        """
        if self.xml_folder:
            xml_path = Path(self.xml_folder) / f'{xml_filename}.xml'
            return self._read_external_template(xml_path)
        return self._read_internal_template(xml_filename)

    def _read_external_template(self, xml_path):
        """读取外部XML模板文件
        
        Args:
            xml_path: XML文件完整路径
            
        Returns:
            str: XML文件内容
        """
        with open(xml_path, 'r', encoding='utf-8') as file:
            return file.read()

    def _read_internal_template(self, xml_filename):
        """读取Word文档内部的XML模板
        
        Args:
            xml_filename: XML文件名
            
        Returns:
            str: XML文件内容
        """
        return self[f'word/{xml_filename}.xml'].decode()

    def render_xml(self, xml_filename, data):
        try:
            xml_template = self.get_xml_template(xml_filename)
            xml_string = self.render_template(xml_template, data).encode()
            self[f'word/{xml_filename}.xml'] = xml_string
            return xml_string
        except FileNotFoundError as e:
            print(f"File not found: {e}")
        except Exception as e:
            print(f"An error occurred: {e}")

    def render_and_add_xml(self, xml_type, data, relation_id=None):
        """渲染并添加新的XML文件到文档
        
        Args:
            xml_type: XML类型
            data: 渲染数据
            relation_id: 可选的关系ID
            
        Returns:
            str: 生成的关系ID
        """
        xml_content = self.render_template(
            self.get_xml_template(xml_type),
            data
        )
        return self.add_xml(xml_type, xml_content, relation_id)

    def render(self, data):
        for xml_type in ('header', 'footer', 'document'):
            self.render_xml(xml_type, data)
        return self


from xl_word.word_file import WordFile 
from xl_word.sheet import Sheet
from lxml import etree
import os
import json
import time
import re

class SuperWordFile(WordFile):
    testing_word = ''  
    testing_xml = ''  

    def __init__(self, folder, file):
        path = os.path.join(folder, file)
        super().__init__(path)
        self.folder = folder

    @staticmethod
    def timestamp():
        return str(int(time.time()))

    def super_replace(self, file):
        path = os.path.join(self.folder, file)
        if os.path.exists(path):
            with open(path, 'rb') as f:
                file_bytes = f.read()
                self.replace(f'word/{file}', file_bytes)

    def super_save(self, file):
        path = os.path.join(self.folder, file)
        self.save(path)

    def test(self):
        """测试文件"""
        python_file_path = os.path.join(self.folder, 'test.py')
        print(f'python {python_file_path}')
        os.system(f'python {python_file_path}')

    def extract(self, res_path):
        """提取XML文件"""
        dst_path = os.path.join(self.folder, f'{self.timestamp()}{res_path}')
        with open(dst_path, 'wb') as f:
            xml_string = self[f'word/{res_path}']
            root = etree.fromstring(xml_string)
            pretty_xml = etree.tostring(root, pretty_print=True, encoding='unicode', xml_declaration=True)
            pretty_xml = pretty_xml.replace('\t', '    ')
            match = re.search(r'<w:body>(.*?)</w:body>', pretty_xml, re.DOTALL)
            if match:
                body_content = match.group(1)
                print(body_content)

            f.write(body_content.encode())

    @staticmethod
    def get_resource_path(relative_path):
        if hasattr(sys, "_MEIPASS"):
            return os.path.join(sys._MEIPASS, relative_path)
        return os.path.join(relative_path)

    @classmethod
    def word2xml(cls, orientation='h'):
        if cls.testing_word:
            cls.extract_document(orientation)
        else:
            cls.new_word(orientation)

    @classmethod
    def xml2word(cls, orientation='h'):
        if cls.testing_xml:
            cls.create_word(orientation)
        else:
            cls.new_xml(orientation)

    @classmethod
    def new_word(cls, orientation):
        docx_file = 'h.docx' if orientation == 'h' else 'v.docx'
        docx_path = cls.get_resource_path(docx_file)
        save_path = f'd:/tmp/{cls.timestamp()}test.docx'
        wf = WordFile(docx_path)
        wf.save(save_path)
        os.startfile(save_path)
        cls.testing_word = save_path

    @classmethod
    def extract_document(cls, orientation):
        wf = WordFile(cls.testing_word)
        save_path = f'd:/tmp/{cls.timestamp()}document.xml'
        with open(save_path, 'wb') as f:
            content = wf['word/document.xml'].decode()
            pattern = r'<w:body>(.*?)<w:sectPr'
            match = re.search(pattern, content, re.DOTALL)
            content = match.group(1)
            f.write(content.encode())
        os.startfile(save_path)
        cls.testing_word = ''

    @classmethod
    def new_xml(cls, orientation):
        docx_file = 'h.docx' if orientation == 'h' else 'v.docx'
        docx_path = cls.get_resource_path(docx_file)
        wf = WordFile(docx_path)
        save_path = f'd:/tmp/{cls.timestamp()}document.xml'
        with open(save_path, 'wb') as f:
            content = wf['word/document.xml']
            f.write(content)
        os.startfile(save_path)
        cls.testing_xml = save_path 

    @classmethod
    def create_word(cls, orientation):
        docx_file = 'h.docx' if orientation == 'h' else 'v.docx'
        docx_path = cls.get_resource_path(docx_file)
        wf = WordFile(docx_path)
        with open(cls.testing_xml, 'rb') as f:
            content = f.read()
            wf['word/document.xml'] = content
            save_path = f'd:/tmp/{cls.timestamp()}test.docx'
            wf.save(save_path)
            os.startfile(save_path)
            cls.testing_xml = ''

    

    
    
if __name__ == '__main__':
    wf = SuperWordFile(r'd:\git\lims\template\inspect\record - 副本','template.docx')
    wf.extract('document.xml')
    exit()
    wf = SuperWordFile('.','123.docx')
    # wf.extract('document.xml')
    # wf.write('document.xml', 'document2.xml')


from xl_word.utils.tree import Tree, E
import random
from lxml import etree


class RelationMixin:
    """处理Word文档中的资源关系映射"""

    RELATIONSHIP_NAMESPACE = "http://schemas.openxmlformats.org/package/2006/relationships"
    OFFICE_RELATIONSHIP_PREFIX = "http://schemas.openxmlformats.org/officeDocument/2006/relationships"
    DEFAULT_RELATIONS_XML = f"""<?xml version="1.0" encoding="UTF-8" standalone="yes"?>
        <Relationships xmlns="{RELATIONSHIP_NAMESPACE}"></Relationships>"""

    def _generate_random_id(self):
        """生成随机的关系ID"""
        return random.randint(1000, 9999)

    def _get_relation_path(self, xml_file):
        """构建关系文件路径
        
        Args:
            xml_file: XML文件名
            
        Returns:
            str: 关系文件完整路径
        """
        return f'word/_rels/{xml_file}.rels'

    def _build_relationship_type(self, relation_type):
        """构建完整的关系类型URI
        
        Args:
            relation_type: 关系类型名称
            
        Returns:
            str: 完整的关系类型URI
        """
        return f"{self.OFFICE_RELATIONSHIP_PREFIX}/{relation_type}"

    def get_relations(self, xml_file):
        """获取指定xml文件的资源映射
        
        Args:
            xml_file: XML文件名,如 'document.xml'
            
        Returns:
            bytes: 关系文件内容
        """
        return self[self._get_relation_path(xml_file)]

    def write_relations(self, xml_file, relations):
        """写入指定xml文件的资源映射
        
        Args:
            xml_file: XML文件名
            relations: 要写入的关系内容
        """
        self[self._get_relation_path(xml_file)] = relations

    def append_relation(self, xml_file, relation_type, relation_target, relation_id=None):
        """添加单条资源映射
        
        Args:
            xml_file: XML文件名,如 'document.xml'
            relation_type: 关系类型,如 'footer'
            relation_target: 目标文件路径
            relation_id: 可选的关系ID
            
        Returns:
            str: 生成的关系ID标识符
        """
        relations = self.get_relations(xml_file) or self.DEFAULT_RELATIONS_XML.encode()
        relation_tree = Tree(relations)
        
        relation_id = relation_id or self._generate_random_id()
        rid = f'rId{relation_id}'
        
        relation_type = self._build_relationship_type(relation_type)
        relation_tree += E.Relationship(Id=rid, Type=relation_type, Target=relation_target)
        
        self.write_relations(xml_file, bytes(relation_tree))
        return rid

    def merge_relations(self, relations_a, relations_b):
        """合并两个关系文件的内容
        
        Args:
            relations_a: 主关系文件内容
            relations_b: 要合并的关系列表
            
        Returns:
            bytes: 合并后的关系文件内容
        """
        relation_tree = etree.fromstring(relations_a)
        for relation in relations_b:
            relation_element = E.Relationship(
                Id=relation['id'],
                Type=relation['type'],
                Target=relation['target']
            )
            relation_tree.append(relation_element)
        return etree.tostring(relation_tree)



from xl_word.utils.fake_zip import FakeZip
from xl_word.utils.tree import Tree, E
from xl_word.mixins import RelationMixin


class WordFile(RelationMixin, FakeZip):
    '''Word文档对象'''
    def __init__(self, file_path):
        super().__init__(file_path)

    def add_image(self, image_bytes, filename=None, xml_filename='document.xml'):
        image_id = self._generate_random_id()
        image_filename = filename or f'{image_id}.jpeg'
        self[f'word/media/{image_filename}'] = image_bytes
        
        rid = self.append_relation(
            xml_filename,
            'image',
            f'media/{image_filename}',
            image_id
        )
        return rid
    
    def add_xml(self, xml_type, xml_content, relation_id=None):
        '''添加xml文件
        
        Args:
            xml_type: xml文件类型
            xml_content: xml文件内容
            relation_id: 关系ID，如果为None则自动生成
            
        Returns:
            str: 生成的关系ID，格式为'rId{xml_id}'
        '''
        xml_id = relation_id or self._generate_random_id()
        xml_filename = f'{xml_type}{xml_id}.xml'
        self.register_xml(xml_type, xml_filename)
        self[f'word/{xml_filename}'] = xml_content
        self.append_relation('document.xml', xml_type, xml_filename, xml_id)
        return f'rId{xml_id}'

    def register_xml(self, xml_type, xml_filename):
        '''注册xml类型到Content_Types.xml
        
        Args:
            xml_type: xml文件类型
            xml_filename: xml文件名
            
        Returns:
            self: 返回实例本身以支持链式调用
        '''
        xml_path = f'/word/{xml_filename}'
        content_type_tree = Tree(self['[Content_Types].xml'])
        content_type = f'application/vnd.openxmlformats-officedocument.wordprocessingml.{xml_type}+xml'
        content_type_tree += E.Override(PartName=xml_path, ContentType=content_type)
        self['[Content_Types].xml'] = bytes(content_type_tree)
        return self
